% Accompanying matlab code for
%
% Hybrid analog-digital beamforming for single-anchor multipath-assisted positioning
% WCNC 2018
%
% this matlab file generates Figures 3 and 4. Important equations are cross
% referenced with the equation numbers in the paper.
%
% [1] Josef Kulmer, Stefan Grebien, Michael Rath, Klaus Witrisal:
%      Hybrid analog-digital beamforming for single-anchor
%      multipath-assisted positioning, WCNC 2018

%% SETUP

SNR=30;
rng(1);
c = 299792458;
Nant=4; % 4 antennas
N_pos = 500; % Montecarlo trials
posvec=[rand(1,N_pos)*4+1;rand(1,N_pos)*6+1];  % position estimates, randomly generated
position_rms_error=zeros(N_pos,1); % lower bound on expected position rms error

ML_resolution=0.25; % evaluate likelihood function at grid with x and y spacing of 25cm

disp(N_pos)
disp(SNR)

%% signal parameters (see Section Results in [1])
md.beta=0.9;
md.Tp=2.4e-9;
md.fc=5.4e9;
md.type='RC';
md.Ts=1.0016e-9;

%% create transmitted signal s[n]
d_max=30; % cut CIR to 30m
mdev.mClk=1/(md.Ts);
time.Ts_raw = 1/(mdev.mClk);
time.N_sig_samp_raw = 4095;
time.tau_raw = ((0:time.N_sig_samp_raw-1)./(mdev.mClk)).';
time.Ts = md.Ts;
time.tau_OS_full =(time.tau_raw(1):time.Ts:(time.tau_raw(end))).';
time.N_sig_samp = d_max/c/time.Ts; %
time.N_sig_samp = floor((time.N_sig_samp-1)/2)*2+1;
time.tau = time.tau_OS_full(1:time.N_sig_samp);

% generated signal s[n], Eq. (3) in [1]
[pulse,dpulse] = generatePulse(md,time.tau_OS_full(round(length(time.tau_OS_full)/2)), time.tau_OS_full,3); % baseband
% limit pulse length for faster calculation
pulse=ifftshift(pulse(round(length(time.tau_OS_full)/2) +(-(time.N_sig_samp-1)/2 : (time.N_sig_samp-1)/2)));
dpulse=ifftshift(dpulse(round(length(time.tau_OS_full)/2) +(-(time.N_sig_samp-1)/2 : (time.N_sig_samp-1)/2)));
Pulse=fft(pulse);


%% Geometry
% synthetic signal
S=[0,0,0,8;
    0,8,6,8;
    6,8,6,0;
    6,0,0,0];

BS=[1.05;1.05]; % position der Basestations

% calculate geometry
R=[0,-1;1,0];
for iS=1:size(S,1)
    Stemp=S(iS,:);
    etemp=[Stemp(3)-Stemp(1);Stemp(4)-Stemp(2)];
    est=R*etemp/norm(etemp);    % e_s'
    Rs=est*est';
    fptemp.Rs{iS}=Rs;
end
fptemp.segments=S;

%% assemble beampattern
[~,beamp]=getbeampattern(md);
beamp(:,2)=beamp(:,2)/max(abs(beamp(:,2)));
% extend vector for simplified interpolation
beamp=[[beamp(:,1)-360;beamp(:,1);beamp(:,1)+360],...
    repmat(beamp(:,2:3),3,1)];


%% evaluated agent positions \tilde{p} (see Section Results in [1])
pevalx=[0.1:ML_resolution:5.9];
pevaly=[0.1:ML_resolution:7.9];
for ix=1:length(pevalx)
    for iy=1:length(pevaly)
        poseval(:,(ix-1)*length(pevaly)+iy)=[pevalx(ix),pevaly(iy)]; % = \tilde{\mathbf{p}} in [1]
    end
end

antenna_offset = [0    -0.02  0    0.02;
    0.02  0    -0.02 0]; % this offset considers the deviation of the antennas to their orgin point (the antennas need to
    % be physically separated). In the paper, we omitted the antenna_offset for simplier notation.
antenna_azimuth_offset = [-pi/2,pi,+pi/2,0]; % the directions of the antennas

% reserve space
r = zeros(Nant*time.N_sig_samp, N_pos);
LL_coherent = zeros(size(poseval,2),N_pos);
LL_noncoherent = zeros(size(poseval,2),N_pos);
LL_proposed = zeros(size(poseval,2),N_pos);
LL_entirelyiterative = zeros(size(poseval,2),N_pos);

% calculate virtual anchors of the base station, required for rms_error
% calculation
VAofBaseStation = computeAgentVAs(BS, fptemp.Rs, fptemp.segments);

%% monte carlo loop
for l_pos=1:N_pos
    %% Compute VAs of Agent as seen by Anchor
    VA = computeAgentVAs(posvec(:, l_pos), fptemp.Rs, fptemp.segments);
    [D, phi] = computeDistanceAoAatAnchor(BS, VA.VA, antenna_offset, antenna_azimuth_offset);
    
    %% generate synth signal
    b=NaN(Nant,size(VA.VA,2));
    amploss=NaN(Nant,size(VA.VA,2));
    for iVA=1:size(VA.VA,2) % for all reflections k
        phase=exp(1i*2*pi*rand);
        for iAnt=1:Nant % for all antennas m
            % get beampattern value b_m(\phi_k)
            b(iAnt,iVA)=interp1(beamp(:,1),beamp(:,2),phi(iVA,iAnt)/pi*180,'linear');
            % shift pulse s to delay \tau_k s_{\tau_k}
            linphaseg=D(iVA,iAnt)/c/time.Ts;
            linphase=(-(length(pulse)-1)/2 : (length(pulse)-1)/2)'/length(pulse)*2*pi*linphaseg;
            pulsetemplate=ifft(Pulse.*exp(-1i*ifftshift(linphase)));
            % amplitude loss = Friis equation -> 1/distance
            amploss(iAnt,iVA)=1/D(iVA,iAnt);
            % consider reflection, each reflection costs -3dB
            if iVA~=1 %LOS
                amploss(iAnt,iVA)=amploss(iAnt,iVA)/sqrt(2);
            end
            % consider carrier phase
            amploss(iAnt,iVA)=amploss(iAnt,iVA)*phase;
            % create r_m, Eq. (3) in [1]
            r( (iAnt-1)*time.N_sig_samp+(1:time.N_sig_samp), l_pos ) = ...
                r( (iAnt-1)*time.N_sig_samp+(1:time.N_sig_samp), l_pos )+...
                amploss(iAnt,iVA)*b(iAnt,iVA)*pulsetemplate;
        end
    end
    
    % and AWGN, Eq. (3) in [1] (parameters see Section results)
    for iAnt=1:Nant
        noiseawgn=1/sqrt(2)*randn(time.N_sig_samp,1)+1i/sqrt(2)*randn(time.N_sig_samp,1);
        r( (iAnt-1)*time.N_sig_samp+(1:time.N_sig_samp), l_pos )=...
            r( (iAnt-1)*time.N_sig_samp+(1:time.N_sig_samp), l_pos )...
            +sqrt(1*norm(pulsetemplate)^2/(10^(SNR/10)))*noiseawgn;
    end
    
    %% Estimation Process for each hypothesized position
    for ip=1:size(poseval,2)
        % compute VAs of Agent
        VA = computeAgentVAs(poseval(:, ip), fptemp.Rs, fptemp.segments);
        % for selected VAs compute Distance and AoA at Anchor antennas
        [D_sel, phi_sel] = computeDistanceAoAatAnchor(BS, VA.VA, antenna_offset, antenna_azimuth_offset);
        
        % for one position compute log-likelihood
        [LL_coherent(ip, l_pos), LL_noncoherent(ip, l_pos), LL_entirelyiterative(ip, l_pos), LL_proposed(ip, l_pos)] = ...
            calc_ll(r(:, l_pos), time, md, D_sel, phi_sel, beamp);
    end
    % calc position_rms_error (the red line in Figure 6 in [1]
    VA = computeAgentVAs(posvec(:, l_pos), fptemp.Rs, fptemp.segments);
    [D_sel, phi_sel] = computeDistanceAoAatAnchor(BS, VA.VA, antenna_offset, antenna_azimuth_offset);
    AntennaSelektor=1:4;
    position_rms_error(l_pos)=calcRMSerror(time, D_sel, phi_sel, pulse, dpulse, (1*norm(pulsetemplate)^2/(10^(SNR/10))),...
        VAofBaseStation, VA, fptemp,AntennaSelektor,amploss);
    
    if l_pos==1 % Figure 3, note for improved illustration, the Figures in the paper were generated with a dense grid of 2.5cm instead of 10cm
        % normalize log likelihood functions to max=0
        for ipl=1:(size(poseval,2)/length(pevaly))
          LL_1(ipl,:)=LL_coherent((1:length(pevaly))+(ipl-1)*length(pevaly),l_pos);
          LL_2(ipl,:)=LL_noncoherent((1:length(pevaly))+(ipl-1)*length(pevaly),l_pos);
          LL_3(ipl,:)=LL_entirelyiterative((1:length(pevaly))+(ipl-1)*length(pevaly),l_pos);
        end
        LL_1=LL_1-max(max(LL_1));
        LL_2=LL_2-max(max(LL_2));
        LL_3=LL_3-max(max(LL_3));
        
        figure; % Figure 3a
        plot(1,1,'.');hold on
        imagesc(pevalx,pevaly,LL_1'),title('coherent'), colormap('jet'),colorbar
        text(BS(1),BS(2),'Anchor')
        text(posvec(1,l_pos),posvec(2,l_pos),'Agent')
        figure; % Figure 3b
        plot(1,1,'.');hold on
        imagesc(pevalx,pevaly,LL_2'),title('non-coherent'), colormap('jet'),colorbar
        text(BS(1),BS(2),'Anchor')
        text(posvec(1,l_pos),posvec(2,l_pos),'Agent')
        figure; % Figure 3c
        plot(1,1,'.');hold on
        imagesc(pevalx,pevaly,LL_3'),title('proposed'), colormap('jet'),colorbar
        text(BS(1),BS(2),'Anchor')
        text(posvec(1,l_pos),posvec(2,l_pos),'Agent')
    end
    disp([num2str(l_pos),' / ',num2str(N_pos)])
end


%% estimate the position error for Figure 4
d = zeros(N_pos, 3);
for l_pos=1:N_pos
    % search for maximum of the likelihood function
    [~,idxmax]=max(LL_coherent(:,l_pos));
    d(l_pos,1)=norm(poseval(:,idxmax)-posvec(:,l_pos));
    [~,idxmax]=max(LL_noncoherent(:,l_pos));
    d(l_pos,2)=norm(poseval(:,idxmax)-posvec(:,l_pos));
    [~,idxmax]=max(LL_proposed(:,l_pos));
    d(l_pos,3)=norm(poseval(:,idxmax)-posvec(:,l_pos));
end
%
% Figure 4
figure
plot(sort(d(1:N_pos,1)),linspace(0,1,N_pos));
hold on
plot(sort(d(1:N_pos,2)),linspace(0,1,N_pos));
plot(sort(d(1:N_pos,3)),linspace(0,1,N_pos));
plot(sort(position_rms_error(1:N_pos)),linspace(0,1,N_pos));

legend({'coherent','non-coherent','proposed',...
    'rms position error'})
xlabel('position error $\|\mathbf{p}-\hat{\mathbf{p}}\|$','Interpreter','Latex')
ylabel('CDF')

save('publishedcode_Figure3_4','d','position_rms_error')
