% Accompanying matlab code for
%
% Hybrid analog-digital beamforming for single-anchor multipath-assisted positioning
% WCNC 2018
%
% this matlab file generates Figure 5. Important equations are cross
% referenced with the equation numbers in the paper.
%
% [1] Josef Kulmer, Stefan Grebien, Michael Rath, Klaus Witrisal:
%      Hybrid analog-digital beamforming for single-anchor
%      multipath-assisted positioning, WCNC 2017

%% SETUP

rng(1);
c = 299792458;
Nant=4; % 4 antennas

%% signal parameters (see Section Results in [1])
md.beta=0.9;
md.Tp=2.4e-9;
md.fc=5.4e9;
md.type='RC';
md.Ts=1/6.95e9;

ML_resolution=0.25; % evaluate likelihood function at grid with x and y spacing of 25cm

%% create transmitted signal s[n]
load('MeasurementData','time');

%% Geometry of room dimension
% synthetic signal
load('MeasurementData','S');

load('MeasurementData','BS'); % position der Basestations

% calculate geometry
R=[0,-1;1,0];
for iS=1:size(S,1)
    Stemp=S(iS,:);
    etemp=[Stemp(3)-Stemp(1);Stemp(4)-Stemp(2)];
    est=R*etemp/norm(etemp);    % e_s'
    Rs=est*est';
    fptemp.Rs{iS}=Rs;
end
fptemp.segments=S;

%% assemble beampattern
[~,beamp]=getbeampattern(md);
beamp(:,2)=beamp(:,2)/max(abs(beamp(:,2)));
% extend vector for simplified interpolation
beamp=[[beamp(:,1)-360;beamp(:,1);beamp(:,1)+360],...
    repmat(beamp(:,2:3),3,1)];


%% evaluated agent positions \tilde{p} (see Section Results in [1])
pevalx=[0:-ML_resolution:-6.3];
pevaly=[0:ML_resolution:8.1];
for ix=1:length(pevalx)
    for iy=1:length(pevaly)
        poseval(:,(ix-1)*length(pevaly)+iy)=[pevalx(ix),pevaly(iy)];
    end
end
% limit evaluated positions to those which are inside the room
poseval(:,find(poseval(1,:)>0))=[];
poseval(:,find(poseval(1,:)<-6.3))=[];
poseval(:,find((poseval(2,:)+poseval(1,:)/6)<0))=[];
poseval(:,find((poseval(2,:)+poseval(1,:)/6)>7.1))=[];

antenna_offset = [0    -0.02  0    0.02;
    0.02  0    -0.02 0]; % this offset considers the deviation of the antennas to their orgin point (the antennas need to
    %be physically separated). In the paper, we omitted the antenna_offset for simplier notation.
antenna_azimuth_offset = [-pi/2,pi,+pi/2,0]; % the directions of the antennas

% calculate virtual anchors of the base station, required for rms_error
% calculation
VAofBaseStation = computeAgentVAs(BS, fptemp.Rs, fptemp.segments);

load('MeasurementData','MIR'); % load measured data
N_pos=size(MIR,2);

% reserve space
LL_coherent = zeros(size(poseval,2),N_pos);
LL_noncoherent = zeros(size(poseval,2),N_pos);
LL_proposed = zeros(size(poseval,2),N_pos);
LL_entirelyiterative = zeros(size(poseval,2),N_pos);
%% monte carlo loop
for l_pos=1:N_pos
    
    %% Estimation Process for each hypothesized position
    for ip=1:size(poseval,2)
        % compute VAs of Agent
        VA = computeAgentVAs(poseval(:, ip), fptemp.Rs, fptemp.segments);
        % for selected VAs compute Distance and AoA at Anchor antennas
        [D_sel, phi_sel] = computeDistanceAoAatAnchor(BS, VA.VA, antenna_offset, antenna_azimuth_offset);
        
        % for one position compute log-likelihood
        [LL_coherent(ip, l_pos), LL_noncoherent(ip, l_pos), LL_entirelyiterative(ip, l_pos), LL_proposed(ip, l_pos)] = ...
            calc_ll(MIR(:, l_pos), time, md, D_sel, phi_sel, beamp);
    end
    disp([num2str(l_pos),' / ',num2str(N_pos)])
end


%% estimate the position error for Figure 5
load('MeasurementData','p'); % load true positions (for evaluation)
posvec=p;
d = zeros(N_pos, 3);
for l_pos=1:N_pos
    % search for maximum of the likelihood function
    [~,idxmax]=max(LL_coherent(:,l_pos));
    d(l_pos,1)=norm(poseval(:,idxmax)-posvec(:,l_pos));
    [~,idxmax]=max(LL_noncoherent(:,l_pos));
    d(l_pos,2)=norm(poseval(:,idxmax)-posvec(:,l_pos));
    [~,idxmax]=max(LL_proposed(:,l_pos));
    d(l_pos,3)=norm(poseval(:,idxmax)-posvec(:,l_pos));
end
save('publishedcode_Figure5','d')
%
% Figure 5
figure
plot(sort(d(1:N_pos,1)),linspace(0,1,N_pos));
hold on
plot(sort(d(1:N_pos,2)),linspace(0,1,N_pos));
plot(sort(d(1:N_pos,3)),linspace(0,1,N_pos));

legend({'coherent','non-coherent','proposed'})
xlabel('position error $\|\mathbf{p}-\hat{\mathbf{p}}\|$','Interpreter','Latex')
ylabel('CDF')
