function [ log_lh_coherent, log_lh_noncoherent, log_lh_entirelyiterative, log_lh_proposed] ...
    = calc_ll( r, time, md, d_hypo, phi_hypo, beampattern)
% computes the likelihood function for a given observation r and hypothized
% agent postion with MPC delays d_hypo and angles phi_hypo
%
% Graz, 2018
% [1] Josef Kulmer, Stefan Grebien, Michael Rath, Klaus Witrisal:
%      Hybrid analog-digital beamforming for single-anchor
%      multipath-assisted positioning, WCNC 2018


log_lh_coherent = 0;
log_lh_noncoherent = 0;
log_lh_entirelyiterative = 0;
log_lh_joint = 0;
log_lh_proposed = 0;

c = 299792458;
N_mpc = size(phi_hypo, 1);
N_ant = size(phi_hypo, 2);

if(mod(time.N_sig_samp,2)==0)
    warning('Please consider using an odd number for time.N_sig_samp');
end

pulse = generatePulse(md, time.tau((time.N_sig_samp-1)/2), time.tau, 3);
pulse = circshift(pulse, -(time.N_sig_samp-1)/2+1);
Pulse = fft(pulse);

X = zeros(N_ant*time.N_sig_samp, N_mpc); % N_ant*N_sig x N_mpc
b = zeros(N_ant,N_mpc);

%% Alpha computation
normalizingConstant = zeros(N_mpc,N_ant);
alpha_iter = zeros(N_mpc,N_ant);
r_res=r; % residual signal (see Section III C in [1])

% for each reflection k
for i_mpc = 1:N_mpc
    % get beampattern for each antenna m
    b(:,i_mpc) = interp1(beampattern(:, 1), beampattern(:, 2), phi_hypo(i_mpc,:)/pi*180, 'linear');
    for i_ant=1:N_ant % for each antenna
        % align signal s to delay \tau_k
        linphaseg=d_hypo(i_mpc,i_ant)/c/time.Ts;
        linphase=(-(time.N_sig_samp-1)/2 : (time.N_sig_samp-1)/2)'/time.N_sig_samp*2*pi*linphaseg;
        pulsetemplate=ifft(Pulse.*exp(-1i*ifftshift(linphase)));
        
        % create X, Eq. (14) in [1]
        X( (i_ant-1)*time.N_sig_samp+(1:time.N_sig_samp), i_mpc )=...
            b(i_ant,i_mpc)*pulsetemplate;
        
        % calculate constant c, Eq. (18) in [1]
        normalizingConstant(i_mpc,i_ant) = abs(b(i_ant, i_mpc)).^2*norm(pulsetemplate).^2;
        % calc amplitudes for each antenna, iteratively, using Eq. (17) in [1]
        % (see Section III C)
        alpha_iter(i_mpc, i_ant)  = conj(b(i_ant, i_mpc))*pulsetemplate'*r_res((i_ant-1)*time.N_sig_samp+(1:time.N_sig_samp))/(norm(pulsetemplate)^2*abs(b(i_ant,i_mpc))^2);
        r_res((i_ant-1)*time.N_sig_samp+(1:time.N_sig_samp))=r_res((i_ant-1)*time.N_sig_samp+(1:time.N_sig_samp))-...
            alpha_iter(i_mpc, i_ant)*b(i_ant, i_mpc)*pulsetemplate;
    end
end

% calc ampltidues for each antenna using Eq. (9) in [1]
alpha_m = zeros(N_mpc, N_ant);
for i_ant = 1:N_ant
    X_m = X((i_ant-1)*time.N_sig_samp+(1:time.N_sig_samp),:); % see Eq. (4) in [1]
    alpha_m(:, i_ant) = (X_m'*X_m)^(-1)*X_m'*r((i_ant-1)*time.N_sig_samp+(1:time.N_sig_samp)); % Eq. (9) in [1]
end


% calculate wheighted average of amplitudes
alpha_abs=zeros(N_mpc,1);
alpha_abs_2 = zeros(N_mpc,1);
% weighted average, Eq. (23) in [1]
for i_mpc = 1:N_mpc
    alpha_abs(i_mpc)=1/sum(normalizingConstant(i_mpc,:)) * normalizingConstant(i_mpc,:)*abs(alpha_iter(i_mpc,:))'; % amplitudes estimated iteratively
    alpha_abs_2(i_mpc) = 1/sum(normalizingConstant(i_mpc,:)) * normalizingConstant(i_mpc,:)*abs(alpha_m(i_mpc,:))'; % amplitudes estimated jointly
end


%% Log-Likelihood Computation of proposed
for i_ant = 1:N_ant
    log_lh_entirelyiterative = log_lh_entirelyiterative ...
        - norm(r((i_ant-1)*time.N_sig_samp+(1:time.N_sig_samp)) - ...
        X(((i_ant-1)*time.N_sig_samp+(1:time.N_sig_samp)),:)*(alpha_abs.*exp(1i*angle(alpha_iter(:,i_ant)))))^2; % Eq. (22) in [1]
    log_lh_joint = log_lh_joint ...
        - norm(r((i_ant-1)*time.N_sig_samp+(1:time.N_sig_samp)) - ...
        X((i_ant-1)*time.N_sig_samp+(1:time.N_sig_samp),:)*(alpha_abs_2.*exp(1i*angle(alpha_m(:,i_ant)))))^2; % Eq. (22) in [1]
end

%% log likelihood computation of coherent
alpha_coherent = (X'*X)^(-1)*X'*r; % Eq. (16) of [1]
log_lh_coherent = -norm(r-X*alpha_coherent)^2; % Eq. (15) of [1]

%% finally, non-coherent
checkpathoverlap=false;
for i_ant = 1:N_ant % for each antenna
    i_mpc = 1;
    S_noPO = zeros(time.N_sig_samp, 0);
    d_sort=sort(d_hypo(:,i_ant)); % sort delays
    
    linphaseg=d_sort(1)/c/time.Ts; %LOS
    linphase=(-(time.N_sig_samp-1)/2 : (time.N_sig_samp-1)/2)'/time.N_sig_samp*2*pi*linphaseg;
    pulsetemplate=ifft(Pulse.*exp(-1i*ifftshift(linphase)));
    S_noPO = [S_noPO pulsetemplate*b(i_ant, i_mpc)];
    
    for i_sort = 2:N_mpc
        % check if there is a path overlap problem
        if(d_sort(i_sort)- d_sort(i_mpc))> md.Tp*c/4 % oh no! there is pathoverlap. (see Sec. III C in [1])
            % if no, then add signal as column vector to matrix X_m
            i_mpc = i_mpc + 1;
            linphaseg=d_sort(i_sort)/c/time.Ts;
            linphase=(-(time.N_sig_samp-1)/2 : (time.N_sig_samp-1)/2)'/time.N_sig_samp*2*pi*linphaseg;
            pulsetemplate=ifft(Pulse.*exp(-1i*ifftshift(linphase)));
            S_noPO = [S_noPO pulsetemplate*b(i_ant, i_mpc)]; % add MPC to X_m (Eq. (9) in [1])
        end
    end
    if i_mpc~=N_mpc
        checkpathoverlap=true; % this switch is used to decide whether to take the iterative amplitudes or the jointly estimated ones
    end
    alpha = (S_noPO'*S_noPO)^(-1)*S_noPO'*r((i_ant-1)*time.N_sig_samp+(1:time.N_sig_samp)); % Eq. (9) in [1]
    log_lh_noncoherent = log_lh_noncoherent ...
        - norm(r((i_ant-1)*time.N_sig_samp+(1:time.N_sig_samp)) - S_noPO*alpha)^2; % Eq. (7) in [1]
end
% see Section III C in [1]
if checkpathoverlap
    log_lh_proposed = log_lh_entirelyiterative;
else
    log_lh_proposed = log_lh_joint;
end
