function [corrected_ir, corrected_ir_shaped] = correctCIR(ir, r_sys, r_cross, cal_param, mClk, pulse, sig)
%function [corrected_ir, corrected_ir_shaped] =
%                 correctCIR(ir, r_sys, r_cross, cal_param, mClk, pulse)
%
% CIR Calculates the calibrated CIR using the previously obtained system and
% crosstalk impulse responses and calibration options.
% Optionally, a filtering using a given pulse is done.
%
% Inputs:
%    ir: The impulse response to calibrate
%    r_sys: The channel sounder system impulse response
%    r_cross: The channel sounder crosstalk impulse response
%    cal_param: A struct holding the needed parameters
%       - c: speed of light
%       - tx_antenna_len: The length of TX antenna in [m]
%       - rx_antenna_len: The length of RX antenna in [m]
%       - c_antenna: The transmission speed in the antenna
%       - additional_len: Length of additional connectors used in the
%              system response measurement
%       - th: Threshold for r_sys (set samples smaller th to 0)
%              to avoid noise gain
%              (given as percentage of max(abs(r_sys)))
%       - truncate_sys: Boolean to indicate whether the system response
%              should be truncated to win_len_idx samples prior to
%              deconvolution (usually 1 to avoid spurious signals in the
%              system response to have influence)
%       - win_len_idx: Length of the truncated system response (see above).
%              Also, the shaped (filtered) signal will be truncated to this length
%     mClk: The sampling clock of the channel sounder (usually 6.95e9 Hz)
%     pulse: Signal of the same length as ir, if given, a filtering with this
%            signal will be performed.
%     sig: Struct with signal parameters
%
% Outputs:
%     corrected_ir: The corrected impulse response (raw, full meas. bandwidth
%     corrected_ir_shaped: Corrected and filtered signal, filtered with pulse
%
% Authors: Paul Meissner, Erik Leitinger, Manuel Lafer, SPSC Lab, 2014

debug = 0;
equalization = 1; %if 1, zero forcing in frequency domain; if 2, MMSE equalization
scale_to_raw = 0; %if 1, the filtered, corrected signal will be scaled to the 
                  %same abs max as the raw signal

% make all inputs column vectors
ir = ir(:);
r_cross = r_cross(:);

%check parameters
if(nargin > 5)
  M_OS = sig.M_OS;
else
  M_OS = 1;
end

if(M_OS ~= 1)  %upsample all signals
  ir = upsample(ir, M_OS);
  r_cross = upsample(r_cross, M_OS);
  r_sys = upsample(r_sys, M_OS);
end

% length of CIR
N_samples = length(ir);

r_cross_orig = r_cross;  %for debug and for correcting r_sys

%check parameters
if(nargin > 5)  
  %if( length(pulse) ~= length(ir) )
  if( length(pulse) ~= length(sig.tau_OS_full) )
    error('Length of given pulse does not match IR length!')
  else
    %find pulse position
    [~, pulse_pos_idx] = max(abs(pulse));
  end
end

%% if this is a measurement without full (new) calibration measurements (i.e. system and crosstalk
%signal measurements), there will be a synchronization entry in the calibration data, as
%the unknown m-sequence phase offset has to be estimated. In this case, also the calibration
%signals have to be updated.
if(isfield(cal_param, 'recalib_len'))
  recalib_len = cal_param.recalib_len;
  
  len = round(recalib_len/cal_param.c*mClk*M_OS);
  
  %find correct length (could be off-by-one by rounding)
  delays = (-3*M_OS:3*M_OS);  %candidates
  energy_temp = zeros(1,length(delays));
  for i_temp = 1:length(delays)
    ir_corr_temp = ir - circshift(r_cross, len+delays(i_temp));
    energy_temp(i_temp) = sigEnergy(ir_corr_temp);
  end
  [~, min_idx] = min(energy_temp);
  recalib_len_idx = len + delays(min_idx);
  
  %shift crosstalk signal by recalib length
  r_cross = circshift(r_cross, recalib_len_idx);
else   %just set the synchronization distance to zero.
  recalib_len = 0;
end

r_sys_corr = r_sys(:)-r_cross_orig;  %here, subtract the original crosstalk, same time-base as r_sys
% figure, hold on, plot(abs(r_sys)), plot(abs(r_cross), 'k--'), plot(abs(r_cross_orig), 'r--')
% figure; hold on; plot(abs(ir)), plot(abs(r_cross),'r--'), plot(abs(r_sys_corr),'k')
ir_corr = ir - r_cross;  %subtract crosstalk from raw data


%% set samples below threshold to 0
[sys_max, sys_max_idx] = max(abs(r_sys_corr));
% figure, hold on, grid on,  [Pxx, f_plot] = pwelch((r_sys_corr), [], [], [], 4*6.95e9, 'twosided');, plot(f_plot/1e9, 10*log10(Pxx), 'b')
% figure,  plotComplexSig(1:length(r_sys_corr),r_sys_corr)

% [Pxx, f_plot] = pwelch((r_sys_corr), [], [], [], 4*6.95e9, 'twosided');, plot(f_plot/1e9, 10*log10(Pxx), 'r')

if(cal_param.truncate_sys)  %perform truncation of system response
  win_start_add_idx = 10*M_OS;  %number of samples to add to window prior to max
  win_start_idx = sys_max_idx-win_start_add_idx;   
  win_end_idx = sys_max_idx+cal_param.win_len_idx;
  win_mask = zeros(size(r_sys_corr));
  
  %assemble the window to cut out of the system response
  if(win_end_idx > N_samples)  %window too long at the end
    overlen_idx = win_end_idx - N_samples;
    win_mask([1:overlen_idx win_start_idx:N_samples]) = 1;
    
  elseif(win_start_idx < 1)   %window too long at the start
    underlen = N_samples + win_start_idx;
    win_mask([underlen:N_samples 1:win_end_idx]) = 1;
  else
    win_mask((win_start_idx):(win_end_idx)) = 1;
  end
  % figure, hold on, plot(abs(r_sys_corr))
  
  % masking
  r_sys_corr(~win_mask) = 0;  
end
% hold on, plot(abs(r_sys_corr), 'r--')

%% equalization 

% frequency domain (zero forcing)
if(equalization == 1)
  r_sys_corr(abs(r_sys_corr) < (sys_max*cal_param.th)) = 0;

  %hold on, plot(abs(r_sys_corr), 'm-.')

  % fft
  R_sys_corr = fft(r_sys_corr);
  %R_sys_corr = R_sys_corr./max(abs(R_sys_corr));
  R_ir = fft(ir_corr);

  % equalization
  corrected_ir = ifft(R_ir./R_sys_corr);  %not shaped corrected IR

elseif(equalization == 2)
  % correct system delay
  ir_corr_shifted = circshift(ir_corr,-sys_max_idx);
  % calculate equalization filter
  R_sys_correl = toeplitz(r_sys_corr);
  h_MMSE = pinv(R_sys_correl)*r_sys_corr;
  % equalization
  corrected_ir = conv(ir_corr_shifted,h_MMSE);
  corrected_ir = corrected_ir(1:N_samples);
end

%{
  figure,hold on, grid on,
  plot(abs(corrected_ir)),
  plot(abs(ir_corr),'r')
%}
%% corrections and also perform pulse shaping

% shift by antenna delay, because its not part of the system response
ant_delay = -round( ((cal_param.tx_antenna_len + cal_param.rx_antenna_len +...
  cal_param.additional_len)/cal_param.c_antenna + recalib_len/cal_param.c)*mClk*M_OS );

% correct not shaped signal
corrected_ir = circshift(corrected_ir, ant_delay);
% scale corrected ir to have the same maximum as the original ir
if(scale_to_raw)
  ir_max = max(abs(ir));
  cor_ir_max = max(abs(corrected_ir));
  corrected_ir = corrected_ir.*(ir_max/cor_ir_max);
end

if(nargin > 5)
  R_corrected_ir = fft(corrected_ir);
  R_cos_win = fft(pulse);   
  %shaping pulse
  corrected_ir_shaped = ifft( R_corrected_ir.*R_cos_win );  %filtered corrected IR
  % scale corrected ir to have the same maximum as the original ir
  if(scale_to_raw)
    cor_ir_max = max(abs(corrected_ir_shaped));
    corrected_ir_shaped = corrected_ir_shaped.*(ir_max/cor_ir_max);
  end
  %cut out IR
  ir_idx_start = pulse_pos_idx;
  win_len_idx = sig.N_sig_samp;
  
  ir_idx_end = pulse_pos_idx+win_len_idx-1;
  
  if(ir_idx_end > N_samples)  %too long, wrap back around origin
    overlen_idx = ir_idx_end - N_samples;
    ir_idx = [ir_idx_start:N_samples 1:overlen_idx];
  else  %length ok
    ir_idx = ir_idx_start:ir_idx_end;
  end
  corrected_ir_shaped = corrected_ir_shaped(ir_idx);

end


%% Debug
if(debug)
  
  figure
  hold on
  plot(sig.tau_OS_full, abs(ir), 'b-')
  plot(sig.tau_OS_full, abs(r_cross_orig), 'g--')
  plot(sig.tau_OS_full, abs(r_cross), 'c--')
  
  %plot(abs(ir_corr), 'r--')
  plot(sig.tau, abs(corrected_ir), 'k')
  plot(sig.tau, abs(corrected_ir_shaped), 'r--')
  
  legend('Signal (uncorrected)', 'Crosstalk, orig.', 'Crosstalk, shifted', 'Corrected signal', 'Filtered signal' )
  
  keyboard
  
end